/*
 *                          Yeppp! library header
 *                  This file is auto-generated by Peach-Py,
 *       Portable Efficient Assembly Code-generator in Higher-level Python,
 *                   part of the Yeppp! library infrastrure
 *
 * This file is part of Yeppp! library and licensed under 2-clause BSD license.
 *
 * Copyright (C) 2010-2012 Marat Dukhan
 * Copyright (C) 2012-2013 Georgia Institute of Technology
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation are those
 * of the authors and should not be interpreted as representing official policies,
 * either expressed or implied, of the Yeppp! Project.
 */

#pragma once

#include <yepPredefines.h>
#include <yepTypes.h>

#ifdef __cplusplus
	extern "C" {
#endif

/** @defgroup yepCore yepCore.h: basic arithmetic operations. */

/** @name	Vector addition */
/**@{*/
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise sums of unsigned 8-bit integer elements in two arrays, producing an array of unsigned 8-bit integer elements.
 * @param[in]	xPointer	Pointer the first input array of unsigned 8-bit integer elements to be added.
 * @param[in]	yPointer	Pointer the second input array of unsigned 8-bit integer elements to be added.
 * @param[out]	sumPointer	Pointer the output array of unsigned 8-bit integer elements of the pairwise sums.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a sumPointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a sumPointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a sumPointer arguments is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V8uV8u_V8u(const Yep8u* YEP_RESTRICT xPointer, const Yep8u* YEP_RESTRICT yPointer, Yep8u* YEP_RESTRICT sumPointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise sums of unsigned 8-bit integer elements in two arrays, producing an array of unsigned 16-bit integer elements.
 * @param[in]	xPointer	Pointer the first input array of unsigned 8-bit integer elements to be added.
 * @param[in]	yPointer	Pointer the second input array of unsigned 8-bit integer elements to be added.
 * @param[out]	sumPointer	Pointer the output array of unsigned 16-bit integer elements of the pairwise sums.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a sumPointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a sumPointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a sumPointer arguments is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE4.1, SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V8uV8u_V16u(const Yep8u* YEP_RESTRICT xPointer, const Yep8u* YEP_RESTRICT yPointer, Yep16u* YEP_RESTRICT sumPointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise sums of signed 8-bit integer elements in two arrays, producing an array of signed 16-bit integer elements.
 * @param[in]	xPointer	Pointer the first input array of signed 8-bit integer elements to be added.
 * @param[in]	yPointer	Pointer the second input array of signed 8-bit integer elements to be added.
 * @param[out]	sumPointer	Pointer the output array of signed 16-bit integer elements of the pairwise sums.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a sumPointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a sumPointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a sumPointer arguments is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE4.1, SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V8sV8s_V16s(const Yep8s* YEP_RESTRICT xPointer, const Yep8s* YEP_RESTRICT yPointer, Yep16s* YEP_RESTRICT sumPointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise sums of unsigned 16-bit integer elements in two arrays, producing an array of unsigned 16-bit integer elements.
 * @param[in]	xPointer	Pointer the first input array of unsigned 16-bit integer elements to be added.
 * @param[in]	yPointer	Pointer the second input array of unsigned 16-bit integer elements to be added.
 * @param[out]	sumPointer	Pointer the output array of unsigned 16-bit integer elements of the pairwise sums.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a sumPointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a sumPointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a sumPointer arguments is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V16uV16u_V16u(const Yep16u* YEP_RESTRICT xPointer, const Yep16u* YEP_RESTRICT yPointer, Yep16u* YEP_RESTRICT sumPointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise sums of unsigned 16-bit integer elements in two arrays, producing an array of unsigned 32-bit integer elements.
 * @param[in]	xPointer	Pointer the first input array of unsigned 16-bit integer elements to be added.
 * @param[in]	yPointer	Pointer the second input array of unsigned 16-bit integer elements to be added.
 * @param[out]	sumPointer	Pointer the output array of unsigned 32-bit integer elements of the pairwise sums.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a sumPointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a sumPointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a sumPointer arguments is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE4.1, SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V16uV16u_V32u(const Yep16u* YEP_RESTRICT xPointer, const Yep16u* YEP_RESTRICT yPointer, Yep32u* YEP_RESTRICT sumPointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise sums of signed 16-bit integer elements in two arrays, producing an array of signed 32-bit integer elements.
 * @param[in]	xPointer	Pointer the first input array of signed 16-bit integer elements to be added.
 * @param[in]	yPointer	Pointer the second input array of signed 16-bit integer elements to be added.
 * @param[out]	sumPointer	Pointer the output array of signed 32-bit integer elements of the pairwise sums.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a sumPointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a sumPointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a sumPointer arguments is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE4.1, SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V16sV16s_V32s(const Yep16s* YEP_RESTRICT xPointer, const Yep16s* YEP_RESTRICT yPointer, Yep32s* YEP_RESTRICT sumPointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise sums of unsigned 32-bit integer elements in two arrays, producing an array of unsigned 32-bit integer elements.
 * @param[in]	xPointer	Pointer the first input array of unsigned 32-bit integer elements to be added.
 * @param[in]	yPointer	Pointer the second input array of unsigned 32-bit integer elements to be added.
 * @param[out]	sumPointer	Pointer the output array of unsigned 32-bit integer elements of the pairwise sums.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a sumPointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a sumPointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a sumPointer arguments is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V32uV32u_V32u(const Yep32u* YEP_RESTRICT xPointer, const Yep32u* YEP_RESTRICT yPointer, Yep32u* YEP_RESTRICT sumPointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise sums of unsigned 32-bit integer elements in two arrays, producing an array of unsigned 64-bit integer elements.
 * @param[in]	xPointer	Pointer the first input array of unsigned 32-bit integer elements to be added.
 * @param[in]	yPointer	Pointer the second input array of unsigned 32-bit integer elements to be added.
 * @param[out]	sumPointer	Pointer the output array of unsigned 64-bit integer elements of the pairwise sums.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a sumPointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a sumPointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a sumPointer arguments is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE4.1, SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V32uV32u_V64u(const Yep32u* YEP_RESTRICT xPointer, const Yep32u* YEP_RESTRICT yPointer, Yep64u* YEP_RESTRICT sumPointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise sums of signed 32-bit integer elements in two arrays, producing an array of signed 64-bit integer elements.
 * @param[in]	xPointer	Pointer the first input array of signed 32-bit integer elements to be added.
 * @param[in]	yPointer	Pointer the second input array of signed 32-bit integer elements to be added.
 * @param[out]	sumPointer	Pointer the output array of signed 64-bit integer elements of the pairwise sums.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a sumPointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a sumPointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a sumPointer arguments is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE4.1, SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V32sV32s_V64s(const Yep32s* YEP_RESTRICT xPointer, const Yep32s* YEP_RESTRICT yPointer, Yep64s* YEP_RESTRICT sumPointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise sums of unsigned 64-bit integer elements in two arrays, producing an array of unsigned 64-bit integer elements.
 * @param[in]	xPointer	Pointer the first input array of unsigned 64-bit integer elements to be added.
 * @param[in]	yPointer	Pointer the second input array of unsigned 64-bit integer elements to be added.
 * @param[out]	sumPointer	Pointer the output array of unsigned 64-bit integer elements of the pairwise sums.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a sumPointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a sumPointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a sumPointer arguments is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V64uV64u_V64u(const Yep64u* YEP_RESTRICT xPointer, const Yep64u* YEP_RESTRICT yPointer, Yep64u* YEP_RESTRICT sumPointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise sums of single-precision (32-bit) floating-point elements in two arrays, producing an array of single-precision (32-bit) floating-point elements.
 * @param[in]	xPointer	Pointer the first input array of single-precision (32-bit) floating-point elements to be added.
 * @param[in]	yPointer	Pointer the second input array of single-precision (32-bit) floating-point elements to be added.
 * @param[out]	sumPointer	Pointer the output array of single-precision (32-bit) floating-point elements of the pairwise sums.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a sumPointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a sumPointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a sumPointer arguments is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V32fV32f_V32f(const Yep32f* YEP_RESTRICT xPointer, const Yep32f* YEP_RESTRICT yPointer, Yep32f* YEP_RESTRICT sumPointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise sums of double-precision (64-bit) floating-point elements in two arrays, producing an array of double-precision (64-bit) floating-point elements.
 * @param[in]	xPointer	Pointer the first input array of double-precision (64-bit) floating-point elements to be added.
 * @param[in]	yPointer	Pointer the second input array of double-precision (64-bit) floating-point elements to be added.
 * @param[out]	sumPointer	Pointer the output array of double-precision (64-bit) floating-point elements of the pairwise sums.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a sumPointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a sumPointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a sumPointer arguments is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V64fV64f_V64f(const Yep64f* YEP_RESTRICT xPointer, const Yep64f* YEP_RESTRICT yPointer, Yep64f* YEP_RESTRICT sumPointer, YepSize length);
/**@}*/

/** @name	Vector subtraction */
/**@{*/
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise differences of unsigned 8-bit integer elements in two arrays, producing an array of unsigned 8-bit integer elements.
 * @param[in]	xPointer	Pointer the first input array of unsigned 8-bit integer elements to be subtracted from.
 * @param[in]	yPointer	Pointer the second input array of unsigned 8-bit integer elements to be subtracted.
 * @param[out]	differencePointer	Pointer the output array of unsigned 8-bit integer elements of the pairwise differences.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a differencePointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a differencePointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a differencePointer arguments is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V8uV8u_V8u(const Yep8u* YEP_RESTRICT xPointer, const Yep8u* YEP_RESTRICT yPointer, Yep8u* YEP_RESTRICT differencePointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise differences of unsigned 8-bit integer elements in two arrays, producing an array of unsigned 16-bit integer elements.
 * @param[in]	xPointer	Pointer the first input array of unsigned 8-bit integer elements to be subtracted from.
 * @param[in]	yPointer	Pointer the second input array of unsigned 8-bit integer elements to be subtracted.
 * @param[out]	differencePointer	Pointer the output array of unsigned 16-bit integer elements of the pairwise differences.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a differencePointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a differencePointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a differencePointer arguments is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE4.1, SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V8uV8u_V16u(const Yep8u* YEP_RESTRICT xPointer, const Yep8u* YEP_RESTRICT yPointer, Yep16u* YEP_RESTRICT differencePointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise differences of signed 8-bit integer elements in two arrays, producing an array of signed 16-bit integer elements.
 * @param[in]	xPointer	Pointer the first input array of signed 8-bit integer elements to be subtracted from.
 * @param[in]	yPointer	Pointer the second input array of signed 8-bit integer elements to be subtracted.
 * @param[out]	differencePointer	Pointer the output array of signed 16-bit integer elements of the pairwise differences.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a differencePointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a differencePointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a differencePointer arguments is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE4.1, SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V8sV8s_V16s(const Yep8s* YEP_RESTRICT xPointer, const Yep8s* YEP_RESTRICT yPointer, Yep16s* YEP_RESTRICT differencePointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise differences of unsigned 16-bit integer elements in two arrays, producing an array of unsigned 16-bit integer elements.
 * @param[in]	xPointer	Pointer the first input array of unsigned 16-bit integer elements to be subtracted from.
 * @param[in]	yPointer	Pointer the second input array of unsigned 16-bit integer elements to be subtracted.
 * @param[out]	differencePointer	Pointer the output array of unsigned 16-bit integer elements of the pairwise differences.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a differencePointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a differencePointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a differencePointer arguments is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V16uV16u_V16u(const Yep16u* YEP_RESTRICT xPointer, const Yep16u* YEP_RESTRICT yPointer, Yep16u* YEP_RESTRICT differencePointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise differences of unsigned 16-bit integer elements in two arrays, producing an array of unsigned 32-bit integer elements.
 * @param[in]	xPointer	Pointer the first input array of unsigned 16-bit integer elements to be subtracted from.
 * @param[in]	yPointer	Pointer the second input array of unsigned 16-bit integer elements to be subtracted.
 * @param[out]	differencePointer	Pointer the output array of unsigned 32-bit integer elements of the pairwise differences.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a differencePointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a differencePointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a differencePointer arguments is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE4.1, SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V16uV16u_V32u(const Yep16u* YEP_RESTRICT xPointer, const Yep16u* YEP_RESTRICT yPointer, Yep32u* YEP_RESTRICT differencePointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise differences of signed 16-bit integer elements in two arrays, producing an array of signed 32-bit integer elements.
 * @param[in]	xPointer	Pointer the first input array of signed 16-bit integer elements to be subtracted from.
 * @param[in]	yPointer	Pointer the second input array of signed 16-bit integer elements to be subtracted.
 * @param[out]	differencePointer	Pointer the output array of signed 32-bit integer elements of the pairwise differences.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a differencePointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a differencePointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a differencePointer arguments is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE4.1, SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V16sV16s_V32s(const Yep16s* YEP_RESTRICT xPointer, const Yep16s* YEP_RESTRICT yPointer, Yep32s* YEP_RESTRICT differencePointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise differences of unsigned 32-bit integer elements in two arrays, producing an array of unsigned 32-bit integer elements.
 * @param[in]	xPointer	Pointer the first input array of unsigned 32-bit integer elements to be subtracted from.
 * @param[in]	yPointer	Pointer the second input array of unsigned 32-bit integer elements to be subtracted.
 * @param[out]	differencePointer	Pointer the output array of unsigned 32-bit integer elements of the pairwise differences.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a differencePointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a differencePointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a differencePointer arguments is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V32uV32u_V32u(const Yep32u* YEP_RESTRICT xPointer, const Yep32u* YEP_RESTRICT yPointer, Yep32u* YEP_RESTRICT differencePointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise differences of unsigned 32-bit integer elements in two arrays, producing an array of unsigned 64-bit integer elements.
 * @param[in]	xPointer	Pointer the first input array of unsigned 32-bit integer elements to be subtracted from.
 * @param[in]	yPointer	Pointer the second input array of unsigned 32-bit integer elements to be subtracted.
 * @param[out]	differencePointer	Pointer the output array of unsigned 64-bit integer elements of the pairwise differences.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a differencePointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a differencePointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a differencePointer arguments is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE4.1, SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V32uV32u_V64u(const Yep32u* YEP_RESTRICT xPointer, const Yep32u* YEP_RESTRICT yPointer, Yep64u* YEP_RESTRICT differencePointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise differences of signed 32-bit integer elements in two arrays, producing an array of signed 64-bit integer elements.
 * @param[in]	xPointer	Pointer the first input array of signed 32-bit integer elements to be subtracted from.
 * @param[in]	yPointer	Pointer the second input array of signed 32-bit integer elements to be subtracted.
 * @param[out]	differencePointer	Pointer the output array of signed 64-bit integer elements of the pairwise differences.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a differencePointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a differencePointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a differencePointer arguments is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE4.1, SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V32sV32s_V64s(const Yep32s* YEP_RESTRICT xPointer, const Yep32s* YEP_RESTRICT yPointer, Yep64s* YEP_RESTRICT differencePointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise differences of unsigned 64-bit integer elements in two arrays, producing an array of unsigned 64-bit integer elements.
 * @param[in]	xPointer	Pointer the first input array of unsigned 64-bit integer elements to be subtracted from.
 * @param[in]	yPointer	Pointer the second input array of unsigned 64-bit integer elements to be subtracted.
 * @param[out]	differencePointer	Pointer the output array of unsigned 64-bit integer elements of the pairwise differences.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a differencePointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a differencePointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a differencePointer arguments is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V64uV64u_V64u(const Yep64u* YEP_RESTRICT xPointer, const Yep64u* YEP_RESTRICT yPointer, Yep64u* YEP_RESTRICT differencePointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise differences of single-precision (32-bit) floating-point elements in two arrays, producing an array of single-precision (32-bit) floating-point elements.
 * @param[in]	xPointer	Pointer the first input array of single-precision (32-bit) floating-point elements to be subtracted from.
 * @param[in]	yPointer	Pointer the second input array of single-precision (32-bit) floating-point elements to be subtracted.
 * @param[out]	differencePointer	Pointer the output array of single-precision (32-bit) floating-point elements of the pairwise differences.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a differencePointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a differencePointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a differencePointer arguments is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V32fV32f_V32f(const Yep32f* YEP_RESTRICT xPointer, const Yep32f* YEP_RESTRICT yPointer, Yep32f* YEP_RESTRICT differencePointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise differences of double-precision (64-bit) floating-point elements in two arrays, producing an array of double-precision (64-bit) floating-point elements.
 * @param[in]	xPointer	Pointer the first input array of double-precision (64-bit) floating-point elements to be subtracted from.
 * @param[in]	yPointer	Pointer the second input array of double-precision (64-bit) floating-point elements to be subtracted.
 * @param[out]	differencePointer	Pointer the output array of double-precision (64-bit) floating-point elements of the pairwise differences.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a differencePointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a differencePointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a differencePointer arguments is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V64fV64f_V64f(const Yep64f* YEP_RESTRICT xPointer, const Yep64f* YEP_RESTRICT yPointer, Yep64f* YEP_RESTRICT differencePointer, YepSize length);
/**@}*/

/** @name	Vector negation */
/**@{*/
/**
	 * @ingroup	yepCore
	 * @brief	Negates a vector of signed 8-bit integer elements, producing an array of signed 8-bit integer elements.
	 * @param[in]	numberPointer	Pointer the input array of signed 8-bit integer elements to be negated from.
	 * @param[out]	negatedNumberPointer	Pointer the output array of signed 8-bit integer negated elements.
	 * @param[in]	length	The length of the arrays pointed by @a numberPointer and @a negatedNumberPointer.
	 * @retval	#YepStatusOk	The computations finished successfully.
	 * @retval	#YepStatusNullPointer	One of the @a numberPointer or @a negatedNumberPointer arguments is null.
	 * @retval	#YepStatusMisalignedPointer	One of the @a numberPointer or @a negatedNumberPointer arguments is not properly aligned.
	 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Negate_V8s_V8s(const Yep8s* YEP_RESTRICT numberPointer, Yep8s* YEP_RESTRICT negatedNumberPointer, YepSize length);
/**
	 * @ingroup	yepCore
	 * @brief	Negates a vector of signed 16-bit integer elements, producing an array of signed 16-bit integer elements.
	 * @param[in]	numberPointer	Pointer the input array of signed 16-bit integer elements to be negated from.
	 * @param[out]	negatedNumberPointer	Pointer the output array of signed 16-bit integer negated elements.
	 * @param[in]	length	The length of the arrays pointed by @a numberPointer and @a negatedNumberPointer.
	 * @retval	#YepStatusOk	The computations finished successfully.
	 * @retval	#YepStatusNullPointer	One of the @a numberPointer or @a negatedNumberPointer arguments is null.
	 * @retval	#YepStatusMisalignedPointer	One of the @a numberPointer or @a negatedNumberPointer arguments is not properly aligned.
	 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Negate_V16s_V16s(const Yep16s* YEP_RESTRICT numberPointer, Yep16s* YEP_RESTRICT negatedNumberPointer, YepSize length);
/**
	 * @ingroup	yepCore
	 * @brief	Negates a vector of signed 32-bit integer elements, producing an array of signed 32-bit integer elements.
	 * @param[in]	numberPointer	Pointer the input array of signed 32-bit integer elements to be negated from.
	 * @param[out]	negatedNumberPointer	Pointer the output array of signed 32-bit integer negated elements.
	 * @param[in]	length	The length of the arrays pointed by @a numberPointer and @a negatedNumberPointer.
	 * @retval	#YepStatusOk	The computations finished successfully.
	 * @retval	#YepStatusNullPointer	One of the @a numberPointer or @a negatedNumberPointer arguments is null.
	 * @retval	#YepStatusMisalignedPointer	One of the @a numberPointer or @a negatedNumberPointer arguments is not properly aligned.
	 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Negate_V32s_V32s(const Yep32s* YEP_RESTRICT numberPointer, Yep32s* YEP_RESTRICT negatedNumberPointer, YepSize length);
/**
	 * @ingroup	yepCore
	 * @brief	Negates a vector of signed 64-bit integer elements, producing an array of signed 64-bit integer elements.
	 * @param[in]	numberPointer	Pointer the input array of signed 64-bit integer elements to be negated from.
	 * @param[out]	negatedNumberPointer	Pointer the output array of signed 64-bit integer negated elements.
	 * @param[in]	length	The length of the arrays pointed by @a numberPointer and @a negatedNumberPointer.
	 * @retval	#YepStatusOk	The computations finished successfully.
	 * @retval	#YepStatusNullPointer	One of the @a numberPointer or @a negatedNumberPointer arguments is null.
	 * @retval	#YepStatusMisalignedPointer	One of the @a numberPointer or @a negatedNumberPointer arguments is not properly aligned.
	 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Negate_V64s_V64s(const Yep64s* YEP_RESTRICT numberPointer, Yep64s* YEP_RESTRICT negatedNumberPointer, YepSize length);
/**
	 * @ingroup	yepCore
	 * @brief	Negates a vector of single-precision (32-bit) floating-point elements, producing an array of single-precision (32-bit) floating-point elements.
	 * @param[in]	numberPointer	Pointer the input array of single-precision (32-bit) floating-point elements to be negated from.
	 * @param[out]	negatedNumberPointer	Pointer the output array of single-precision (32-bit) floating-point negated elements.
	 * @param[in]	length	The length of the arrays pointed by @a numberPointer and @a negatedNumberPointer.
	 * @retval	#YepStatusOk	The computations finished successfully.
	 * @retval	#YepStatusNullPointer	One of the @a numberPointer or @a negatedNumberPointer arguments is null.
	 * @retval	#YepStatusMisalignedPointer	One of the @a numberPointer or @a negatedNumberPointer arguments is not properly aligned.
	 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Negate_V32f_V32f(const Yep32f* YEP_RESTRICT numberPointer, Yep32f* YEP_RESTRICT negatedNumberPointer, YepSize length);
/**
	 * @ingroup	yepCore
	 * @brief	Negates a vector of double-precision (64-bit) floating-point elements, producing an array of double-precision (64-bit) floating-point elements.
	 * @param[in]	numberPointer	Pointer the input array of double-precision (64-bit) floating-point elements to be negated from.
	 * @param[out]	negatedNumberPointer	Pointer the output array of double-precision (64-bit) floating-point negated elements.
	 * @param[in]	length	The length of the arrays pointed by @a numberPointer and @a negatedNumberPointer.
	 * @retval	#YepStatusOk	The computations finished successfully.
	 * @retval	#YepStatusNullPointer	One of the @a numberPointer or @a negatedNumberPointer arguments is null.
	 * @retval	#YepStatusMisalignedPointer	One of the @a numberPointer or @a negatedNumberPointer arguments is not properly aligned.
	 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Negate_V64f_V64f(const Yep64f* YEP_RESTRICT numberPointer, Yep64f* YEP_RESTRICT negatedNumberPointer, YepSize length);
/**@}*/

/** @name	Vector elementwise multiplication */
/**@{*/
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise products of unsigned 8-bit integer elements in two arrays, producing an array of unsigned 8-bit integer elements.
 * @param[in]	xPointer	Pointer the first input array of unsigned 8-bit integer elements to be multiplied.
 * @param[in]	yPointer	Pointer the second input array of unsigned 8-bit integer elements to be multiplied.
 * @param[out]	productPointer	Pointer the output array of unsigned 8-bit integer elements of the pairwise products.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a productPointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a productPointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a productPointer arguments is not properly aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V8uV8u_V8u(const Yep8u* YEP_RESTRICT xPointer, const Yep8u* YEP_RESTRICT yPointer, Yep8u* YEP_RESTRICT productPointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise products of unsigned 8-bit integer elements in two arrays, producing an array of unsigned 16-bit integer elements.
 * @param[in]	xPointer	Pointer the first input array of unsigned 8-bit integer elements to be multiplied.
 * @param[in]	yPointer	Pointer the second input array of unsigned 8-bit integer elements to be multiplied.
 * @param[out]	productPointer	Pointer the output array of unsigned 16-bit integer elements of the pairwise products.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a productPointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a productPointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a productPointer arguments is not properly aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V8uV8u_V16u(const Yep8u* YEP_RESTRICT xPointer, const Yep8u* YEP_RESTRICT yPointer, Yep16u* YEP_RESTRICT productPointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise products of signed 8-bit integer elements in two arrays, producing an array of signed 16-bit integer elements.
 * @param[in]	xPointer	Pointer the first input array of signed 8-bit integer elements to be multiplied.
 * @param[in]	yPointer	Pointer the second input array of signed 8-bit integer elements to be multiplied.
 * @param[out]	productPointer	Pointer the output array of signed 16-bit integer elements of the pairwise products.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a productPointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a productPointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a productPointer arguments is not properly aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V8sV8s_V16s(const Yep8s* YEP_RESTRICT xPointer, const Yep8s* YEP_RESTRICT yPointer, Yep16s* YEP_RESTRICT productPointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise products of unsigned 16-bit integer elements in two arrays, producing an array of unsigned 16-bit integer elements.
 * @param[in]	xPointer	Pointer the first input array of unsigned 16-bit integer elements to be multiplied.
 * @param[in]	yPointer	Pointer the second input array of unsigned 16-bit integer elements to be multiplied.
 * @param[out]	productPointer	Pointer the output array of unsigned 16-bit integer elements of the pairwise products.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a productPointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a productPointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a productPointer arguments is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V16uV16u_V16u(const Yep16u* YEP_RESTRICT xPointer, const Yep16u* YEP_RESTRICT yPointer, Yep16u* YEP_RESTRICT productPointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise products of unsigned 16-bit integer elements in two arrays, producing an array of unsigned 32-bit integer elements.
 * @param[in]	xPointer	Pointer the first input array of unsigned 16-bit integer elements to be multiplied.
 * @param[in]	yPointer	Pointer the second input array of unsigned 16-bit integer elements to be multiplied.
 * @param[out]	productPointer	Pointer the output array of unsigned 32-bit integer elements of the pairwise products.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a productPointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a productPointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a productPointer arguments is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V16uV16u_V32u(const Yep16u* YEP_RESTRICT xPointer, const Yep16u* YEP_RESTRICT yPointer, Yep32u* YEP_RESTRICT productPointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise products of signed 16-bit integer elements in two arrays, producing an array of signed 32-bit integer elements.
 * @param[in]	xPointer	Pointer the first input array of signed 16-bit integer elements to be multiplied.
 * @param[in]	yPointer	Pointer the second input array of signed 16-bit integer elements to be multiplied.
 * @param[out]	productPointer	Pointer the output array of signed 32-bit integer elements of the pairwise products.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a productPointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a productPointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a productPointer arguments is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V16sV16s_V32s(const Yep16s* YEP_RESTRICT xPointer, const Yep16s* YEP_RESTRICT yPointer, Yep32s* YEP_RESTRICT productPointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise products of unsigned 32-bit integer elements in two arrays, producing an array of unsigned 32-bit integer elements.
 * @param[in]	xPointer	Pointer the first input array of unsigned 32-bit integer elements to be multiplied.
 * @param[in]	yPointer	Pointer the second input array of unsigned 32-bit integer elements to be multiplied.
 * @param[out]	productPointer	Pointer the output array of unsigned 32-bit integer elements of the pairwise products.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a productPointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a productPointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a productPointer arguments is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE4.1, SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V32uV32u_V32u(const Yep32u* YEP_RESTRICT xPointer, const Yep32u* YEP_RESTRICT yPointer, Yep32u* YEP_RESTRICT productPointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise products of unsigned 32-bit integer elements in two arrays, producing an array of unsigned 64-bit integer elements.
 * @param[in]	xPointer	Pointer the first input array of unsigned 32-bit integer elements to be multiplied.
 * @param[in]	yPointer	Pointer the second input array of unsigned 32-bit integer elements to be multiplied.
 * @param[out]	productPointer	Pointer the output array of unsigned 64-bit integer elements of the pairwise products.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a productPointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a productPointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a productPointer arguments is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE4.1, SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V32uV32u_V64u(const Yep32u* YEP_RESTRICT xPointer, const Yep32u* YEP_RESTRICT yPointer, Yep64u* YEP_RESTRICT productPointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise products of signed 32-bit integer elements in two arrays, producing an array of signed 64-bit integer elements.
 * @param[in]	xPointer	Pointer the first input array of signed 32-bit integer elements to be multiplied.
 * @param[in]	yPointer	Pointer the second input array of signed 32-bit integer elements to be multiplied.
 * @param[out]	productPointer	Pointer the output array of signed 64-bit integer elements of the pairwise products.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a productPointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a productPointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a productPointer arguments is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE4.1, SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V32sV32s_V64s(const Yep32s* YEP_RESTRICT xPointer, const Yep32s* YEP_RESTRICT yPointer, Yep64s* YEP_RESTRICT productPointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise products of unsigned 64-bit integer elements in two arrays, producing an array of unsigned 64-bit integer elements.
 * @param[in]	xPointer	Pointer the first input array of unsigned 64-bit integer elements to be multiplied.
 * @param[in]	yPointer	Pointer the second input array of unsigned 64-bit integer elements to be multiplied.
 * @param[out]	productPointer	Pointer the output array of unsigned 64-bit integer elements of the pairwise products.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a productPointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a productPointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a productPointer arguments is not properly aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V64uV64u_V64u(const Yep64u* YEP_RESTRICT xPointer, const Yep64u* YEP_RESTRICT yPointer, Yep64u* YEP_RESTRICT productPointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise products of single-precision (32-bit) floating-point elements in two arrays, producing an array of single-precision (32-bit) floating-point elements.
 * @param[in]	xPointer	Pointer the first input array of single-precision (32-bit) floating-point elements to be multiplied.
 * @param[in]	yPointer	Pointer the second input array of single-precision (32-bit) floating-point elements to be multiplied.
 * @param[out]	productPointer	Pointer the output array of single-precision (32-bit) floating-point elements of the pairwise products.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a productPointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a productPointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a productPointer arguments is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V32fV32f_V32f(const Yep32f* YEP_RESTRICT xPointer, const Yep32f* YEP_RESTRICT yPointer, Yep32f* YEP_RESTRICT productPointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise products of double-precision (64-bit) floating-point elements in two arrays, producing an array of double-precision (64-bit) floating-point elements.
 * @param[in]	xPointer	Pointer the first input array of double-precision (64-bit) floating-point elements to be multiplied.
 * @param[in]	yPointer	Pointer the second input array of double-precision (64-bit) floating-point elements to be multiplied.
 * @param[out]	productPointer	Pointer the output array of double-precision (64-bit) floating-point elements of the pairwise products.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a productPointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a productPointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a productPointer arguments is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V64fV64f_V64f(const Yep64f* YEP_RESTRICT xPointer, const Yep64f* YEP_RESTRICT yPointer, Yep64f* YEP_RESTRICT productPointer, YepSize length);
/**@}*/

/** @name	Vector elementwise multiplication and addition */
/**@{*/
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise products of single-precision (32-bit) floating-point elements in two arrays and then adds the third single-precision (32-bit) floating-point array to the result, producing an array of single-precision (32-bit) floating-point elements.
 * @param[in]	xPointer	Pointer the first input array of single-precision (32-bit) floating-point elements to be multiplied.
 * @param[in]	yPointer	Pointer the second input array of single-precision (32-bit) floating-point elements to be multiplied.
 * @param[in]	zPointer	Pointer the input array of single-precision (32-bit) floating-point elements to be added to the intermediate multiplication result.
 * @param[out]	macPointer	Pointer the output array of single-precision (32-bit) floating-point elements.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, @a zPointer, and @a macPointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, @a zPointer, or @a macPointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, @a zPointer, or @a macPointer arguments is not properly aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_MultiplyAdd_V32fV32fV32f_V32f(const Yep32f* YEP_RESTRICT xPointer, const Yep32f* YEP_RESTRICT yPointer, const Yep32f* YEP_RESTRICT zPointer, Yep32f* YEP_RESTRICT macPointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise products of double-precision (64-bit) floating-point elements in two arrays and then adds the third double-precision (64-bit) floating-point array to the result, producing an array of double-precision (64-bit) floating-point elements.
 * @param[in]	xPointer	Pointer the first input array of double-precision (64-bit) floating-point elements to be multiplied.
 * @param[in]	yPointer	Pointer the second input array of double-precision (64-bit) floating-point elements to be multiplied.
 * @param[in]	zPointer	Pointer the input array of double-precision (64-bit) floating-point elements to be added to the intermediate multiplication result.
 * @param[out]	macPointer	Pointer the output array of double-precision (64-bit) floating-point elements.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, @a zPointer, and @a macPointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, @a zPointer, or @a macPointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, @a zPointer, or @a macPointer arguments is not properly aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_MultiplyAdd_V64fV64fV64f_V64f(const Yep64f* YEP_RESTRICT xPointer, const Yep64f* YEP_RESTRICT yPointer, const Yep64f* YEP_RESTRICT zPointer, Yep64f* YEP_RESTRICT macPointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise products of single-precision (32-bit) floating-point elements in two arrays and then adds the third single-precision (32-bit) floating-point array to the result, overwriting the third array.
 * @param[in]	xPointer	Pointer the first input array of single-precision (32-bit) floating-point elements to be multiplied.
 * @param[in]	yPointer	Pointer the second input array of single-precision (32-bit) floating-point elements to be multiplied.
 * @param[in,out]	zPointer	Pointer the input/output array of single-precision (32-bit) floating-point elements to be added to the intermediate multiplication result.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a zPointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a zPointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a zPointer arguments is not properly aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_MultiplyAdd_V32fS32fIV32f_IV32f(const Yep32f* YEP_RESTRICT xPointer, const Yep32f* YEP_RESTRICT yPointer, Yep32f* YEP_RESTRICT zPointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes pairwise products of double-precision (64-bit) floating-point elements in two arrays and then adds the third double-precision (64-bit) floating-point array to the result, overwriting the third array.
 * @param[in]	xPointer	Pointer the first input array of double-precision (64-bit) floating-point elements to be multiplied.
 * @param[in]	yPointer	Pointer the second input array of double-precision (64-bit) floating-point elements to be multiplied.
 * @param[in,out]	zPointer	Pointer the input/output array of double-precision (64-bit) floating-point elements to be added to the intermediate multiplication result.
 * @param[in]	length	The length of the arrays pointed by @a xPointer, @a yPointer, and @a zPointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a zPointer arguments is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a zPointer arguments is not properly aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_MultiplyAdd_V64fS64fIV64f_IV64f(const Yep64f* YEP_RESTRICT xPointer, const Yep64f* YEP_RESTRICT yPointer, Yep64f* YEP_RESTRICT zPointer, YepSize length);
/**@}*/

/** @name	Dot product */
/**@{*/
/**
 * @ingroup	yepCore
 * @brief	Computes the dot product of single-precision (32-bit) floating-point elements in two arrays.
 * @param[in]	xPointer	Pointer the first input vector of single-precision (32-bit) floating-point elements.
 * @param[in]	yPointer	Pointer the second input vector of single-precision (32-bit) floating-point elements.
 * @param[out]	dotProductPointer	Pointer the output single-precision (32-bit) floating-point variable.
 * @param[in]	length	The length of the arrays pointed by @a xPointer and @a yPointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a dotProductPointer pointers is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a dotProductPointer pointers is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE, SSE3</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>AMD Bulldozer</td><td>FMA4, AVX</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_DotProduct_V32fV32f_S32f(const Yep32f* YEP_RESTRICT xPointer, const Yep32f* YEP_RESTRICT yPointer, Yep32f* YEP_RESTRICT dotProductPointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes the dot product of double-precision (64-bit) floating-point elements in two arrays.
 * @param[in]	xPointer	Pointer the first input vector of double-precision (64-bit) floating-point elements.
 * @param[in]	yPointer	Pointer the second input vector of double-precision (64-bit) floating-point elements.
 * @param[out]	dotProductPointer	Pointer the output double-precision (64-bit) floating-point variable.
 * @param[in]	length	The length of the arrays pointed by @a xPointer and @a yPointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	One of the @a xPointer, @a yPointer, or @a dotProductPointer pointers is null.
 * @retval	#YepStatusMisalignedPointer	One of the @a xPointer, @a yPointer, or @a dotProductPointer pointers is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE3, SSE2</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>AMD Bulldozer</td><td>FMA4, AVX</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Bonnell</td><td>SSE, SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_DotProduct_V64fV64f_S64f(const Yep64f* YEP_RESTRICT xPointer, const Yep64f* YEP_RESTRICT yPointer, Yep64f* YEP_RESTRICT dotProductPointer, YepSize length);
/**@}*/

/** @name	Summation of squares (squared L2 norm) */
/**@{*/
/**
 * @ingroup	yepCore
 * @brief	Computes the sum of squares of single-precision (32-bit) floating-point elements in the input array.
 * @param[in]	numberPointer	Pointer the input array of single-precision (32-bit) floating-point elements.
 * @param[out]	sumSquaresPointer	Pointer the single-precision (32-bit) floating-point element to hold the sum of squares.
 * @param[in]	length	The length of the array pointed by @a numberPointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	@a numberPointer or @a sumSquaresPointer argument is null.
 * @retval	#YepStatusMisalignedPointer	@a numberPointer or @a sumSquaresPointer argument is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE, SSE3</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>AMD Bulldozer</td><td>FMA4, AVX</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_SumSquares_V32f_S32f(const Yep32f* YEP_RESTRICT numberPointer, Yep32f* YEP_RESTRICT sumSquaresPointer, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes the sum of squares of double-precision (64-bit) floating-point elements in the input array.
 * @param[in]	numberPointer	Pointer the input array of double-precision (64-bit) floating-point elements.
 * @param[out]	sumSquaresPointer	Pointer the double-precision (64-bit) floating-point element to hold the sum of squares.
 * @param[in]	length	The length of the array pointed by @a numberPointer.
 * @retval	#YepStatusOk	The computations finished successfully.
 * @retval	#YepStatusNullPointer	@a numberPointer or @a sumSquaresPointer argument is null.
 * @retval	#YepStatusMisalignedPointer	@a numberPointer or @a sumSquaresPointer argument is not properly aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE3, SSE2</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>AMD Bulldozer</td><td>FMA4, AVX</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_SumSquares_V64f_S64f(const Yep64f* YEP_RESTRICT numberPointer, Yep64f* YEP_RESTRICT sumSquaresPointer, YepSize length);
/**@}*/

#ifdef __cplusplus
	} // extern "C"
#endif